require 'faraday'
require 'faraday_middleware'
require 'typhoeus/adapters/faraday'
require 'virtus'
require 'active_model'

require_relative 'client/version'
require_relative 'client/validation'
require_relative 'client/item'
require_relative 'client/address'
require_relative 'client/order'
require_relative 'client/size'
require_relative 'client/article'
require_relative 'client/style'
require_relative 'client/color'
require_relative 'client/program'
require_relative 'client/look_item'
require_relative 'client/look'
require_relative 'client/collection'

module Bliss
  module Client
    module_function

    ADAPTER = :typhoeus
    API_VERSION = ENV['BLISS_API_VERSION'] || 'v1'
    TOKEN_PATH = '/oauth/token'

    def connection
      @connection = nil if test?

      @connection ||= Faraday.new(url: api_url) do |f|
        f.request :oauth2, token unless skip_auth?
        f.request :json
        f.response :json, content_type: /\bjson$/
        if debug?
          f.response :logger
        end
        f.adapter ADAPTER
      end
    end

    def token
      # Fetch the token every time when testing, so that VCR can expect this
      # request to happen.
      @token = nil if test?

      @token ||= begin
        connection = Faraday.new(url: server) do |f|
          f.request :json
          f.response :json, content_type: /\bjson$/
          if debug?
            f.response :logger
          end
          f.adapter ADAPTER
        end

        response = connection.post(
          TOKEN_PATH,
          grant_type: 'client_credentials',
          client_id: client_id,
          client_secret: client_secret
        )

        response.body.fetch('access_token')
      end
    end

    def api_url
      "#{server}/api/#{API_VERSION}"
    end

    def server
      ENV['BLISS_SERVER']
    end

    def client_id
      ENV['BLISS_CLIENT_ID']
    end

    def client_secret
      ENV['BLISS_CLIENT_SECRET']
    end

    def debug?
      ENV['BLISS_CLIENT_DEBUG']
    end

    def test?
      ENV['BLISS_CLIENT_ENV'] == 'test'
    end

    def skip_auth?
      ENV['BLISS_CLIENT_SKIP_AUTH'] == 'true'
    end
  end
end
