module Bliss
  module Client
    class Order
      include Virtus.model
      include Validation

      attribute :id, Integer
      attribute :type, String
      attribute :items, Array[Item]
      attribute :delivery_address, Address
      attribute :remote_id, Integer
      attribute :blizzard_customer_id, Integer
      attribute :created_at, DateTime

      validates_presence_of :type, :items, :delivery_address,
        :remote_id, :blizzard_customer_id

      def self.create(attributes)
        order = new(attributes)
        order.create
        order
      end

      def create
        validate!

        response = Client.connection.post(
          'orders',
          blizzard_customer_id: blizzard_customer_id,
          type: type,
          items: items.map(&:attributes),
          delivery_address: delivery_address.attributes,
          remote_id: remote_id
        )

        if response.success?
          body = response.body
          self.id = body.fetch('id')
          self.created_at = body.fetch('created_at')
        else
          raise JSON.parse(response.body).fetch('message')
        end
      end

      def validate!
        super
        items.each &:validate!
        delivery_address.validate!
      end
    end
  end
end
